using System;
using System.Drawing;
using System.Collections.Generic;
using System.Text;
using System.Windows.Forms;
using System.Runtime.InteropServices;

namespace ADX_IE_NetControl_CS
{
    class MyCustomToolStrip : ToolStrip
    {
        internal IntPtr hThemeModule = IntPtr.Zero;
        internal DllIsThemeActive IsThemeActive = null;
        internal DllDrawThemeParentBackground DrawThemeParentBackground = null;

        internal delegate bool DllIsThemeActive();
        internal delegate void DllDrawThemeParentBackground(IntPtr hWnd, IntPtr hDC, ref RECT rect);

        [ComVisible(false), StructLayout(LayoutKind.Sequential, Pack = 4)]
        public struct RECT
        {
            public int Left;
            public int Top;
            public int Right;
            public int Bottom;

            public RECT(int left, int top, int right, int bottom)
            {
                Left = left;
                Top = top;
                Right = right;
                Bottom = bottom;
            }

            public RECT(Rectangle rect)
            {
                Left = rect.Left;
                Top = rect.Top;
                Right = rect.Right;
                Bottom = rect.Bottom;
            }

            public Rectangle ToRectangle()
            {
                return new Rectangle(Left, Top, Right, Bottom - 1);
            }
        }

        [DllImport("kernel32.dll")]
        public static extern bool FreeLibrary(IntPtr hModule);

        [DllImport("kernel32.dll")]
        public static extern IntPtr LoadLibrary(string lpFileName);

        [DllImport("kernel32.dll", CharSet = CharSet.Ansi)]
        public static extern IntPtr GetProcAddress(IntPtr hModule, string lpProcName);

        public MyCustomToolStrip()
        {
            if (!this.DesignMode)
            {
                this.DoubleBuffered = true;

                hThemeModule = LoadLibrary("UxTheme.dll");
                if (hThemeModule != IntPtr.Zero)
                {
                    IntPtr dllIsThemeActive =
                        GetProcAddress(hThemeModule, "IsThemeActive");
                    if (dllIsThemeActive != IntPtr.Zero)
                    {
                        IsThemeActive =
                            (DllIsThemeActive)Marshal.GetDelegateForFunctionPointer(
                            dllIsThemeActive, typeof(DllIsThemeActive));
                    }

                    IntPtr dllDrawThemeParentBackground =
                        GetProcAddress(hThemeModule, "DrawThemeParentBackground");
                    if (dllDrawThemeParentBackground != IntPtr.Zero)
                    {
                        DrawThemeParentBackground =
                           (DllDrawThemeParentBackground)Marshal.GetDelegateForFunctionPointer(
                            dllDrawThemeParentBackground, typeof(DllDrawThemeParentBackground));
                    }

                    Application.EnableVisualStyles();
                    Application.DoEvents();
                }
            }
        }

        protected override void Dispose(bool disposing)
        {
            if (disposing)
            {
                if (hThemeModule != IntPtr.Zero)
                {
                    FreeLibrary(hThemeModule);
                    hThemeModule = IntPtr.Zero;
                }
            }
            base.Dispose(disposing);
        }

        protected override void OnPaintBackground(PaintEventArgs e)
        {
            if (hThemeModule != IntPtr.Zero && !DesignMode)
            {
                if (IsThemeActive())
                {
                    IntPtr hdc = IntPtr.Zero;
                    RECT rect = new RECT(0, 0, Width, Height);

                    hdc = e.Graphics.GetHdc();
                    if (hdc != IntPtr.Zero)
                        try
                        {
                            DrawThemeParentBackground(this.Handle, hdc, ref rect);
                        }
                        finally
                        {
                            e.Graphics.ReleaseHdc(hdc);
                        }
                    return;
                }
            }
            base.OnPaintBackground(e);
        }

        protected override void OnPaint(PaintEventArgs e)
        {
            base.OnPaint(e);
            if (hThemeModule != IntPtr.Zero && !DesignMode)
            {
                if (IsThemeActive())
                {
                    IntPtr hdc = IntPtr.Zero;
                    RECT rect = new RECT(0, Height - 2, Width, Height);

                    hdc = e.Graphics.GetHdc();
                    if (hdc != IntPtr.Zero)
                        try
                        {
                            DrawThemeParentBackground(this.Handle, hdc, ref rect);
                        }
                        finally
                        {
                            e.Graphics.ReleaseHdc(hdc);
                        }
                }
            }
        }

    }
}